#include <cmath>

#include "hardware/adc.h"
#include "pico/stdlib.h"
#include "tusb.h"

#include "utils.h"

uint8_t calcWPM(float percent, uint8_t wpmMin, uint8_t wpmMax)
{
    static float lastPercent {0};
    static float lastResult {0};

    auto wpm = (percent * (wpmMax - wpmMin) / 100) + wpmMin;
    uint8_t result = static_cast<uint8_t>(std::round(wpm));

    float amountChanged = std::abs(percent - lastPercent);

    // Only change WPM if poti value change was big enough
    // (Preventing rapid WPM jumps back and forth.)
    if (amountChanged > 0.6) {
        lastPercent = percent;
        lastResult = result;
        return result;
    } else {
        return lastResult;
    };
}

float potiRead()
{
    // 12-bit conversion, assume max value == ADC_VREF == 3.3 V
    const float conversion_factor = 3.3f / (1 << 12);
    float voltage = adc_read() * conversion_factor;

    return voltage * 100 / 3.3;
}

void usbSend(uint8_t itf, uint8_t buf[], uint32_t count)
{
    for (uint32_t i = 0; i < count; i++) {
        tud_cdc_n_write_char(itf, buf[i]);
    }

    tud_cdc_n_write_flush(itf);
}

void usbSend(uint8_t itf, uint8_t value)
{
    tud_cdc_n_write_char(itf, value);
    tud_cdc_n_write_flush(itf);
}